//******************************************************************************
//* MiniCISC CPU                                                               *
//*                                                                            *
//* Arithmetic and logic unit (ALU).                                           *
//******************************************************************************
module alu(
   //Clock signal.
   input  wire       clk,
   
   //Signals related to the function selection.
   input  wire [1:0] op_type,       //Operation type select signal
   input  wire       add_sub,       //Arithmetic operation select signal
   input  wire [1:0] logic_op,      //Logic operation select signal
   input  wire [1:0] shift_op,      //Shift operation select signal
   
   //Input operands.
   input  wire [7:0] operand1,      //ALU operand 1
   input  wire [7:0] operand2,      //ALU operand 2
   input  wire       shift_in,      //Bit to be shifted in.
   
   //ALU result.
   output wire [7:0] result,
   
   //Flags.
   output reg        flag_z,        //Zero flag
   output reg        flag_c         //Carry flag
);

`include "src\control_defs.vh"
`include "src\opcode_defs.vh"

//******************************************************************************
//* Executing the arithmetic operations (addition, subtraction).               *
//* The MSb of the result is the carry bit.                                    *
//******************************************************************************
reg [8:0] addsub_result;

always @(*)
begin
   if (add_sub == ARITH_OP_ADD)
      addsub_result = {1'b0, operand1} + {1'b0, operand2};  //Addition
   else
      addsub_result = {1'b0, operand1} - {1'b0, operand2};  //Subtraction
end


//******************************************************************************
//* Executing the logic operations (bitwise AND, OR, XOR).                     *
//******************************************************************************
reg [7:0] logic_result;

always @(*)
begin
   case (logic_op)
      LOGIC_OP_AND: logic_result = operand1 & operand2;     //Bitwise AND
      LOGIC_OP_OR : logic_result = operand1 | operand2;     //Bitwise OR
      LOGIC_OP_XOR: logic_result = operand1 ^ operand2;     //Bitwise XOR
      default     : logic_result = 8'd0;
   endcase
end


//******************************************************************************
//* Executing the shift operations.                                            *
//******************************************************************************
reg [8:0] shift_result;

always @(*)
begin
   case (shift_op)
      //Shift left
      SHIFT_SHL: shift_result = {operand1[7], operand1[6:0], shift_in};
      //Shift right
      SHIFT_SHR: shift_result = {operand1[0], shift_in, operand1[7:1]};
      //Rotate left
      SHIFT_ROL: shift_result = {operand1[7], operand1[6:0], operand1[7]};
      //Rotate right
      SHIFT_ROR: shift_result = {operand1[0], operand1[0], operand1[7:1]};
   endcase
end


//******************************************************************************
//* Output select MUX.                                                         *
//******************************************************************************
reg [8:0] alu_result;

always @(*)
begin
   case (op_type)
      ALU_OP_MOVE : alu_result = {1'b0, operand1};          //Register move
      ALU_OP_ARITH: alu_result = addsub_result;             //Arithmetic op.
      ALU_OP_LOGIC: alu_result = {1'b0, logic_result};      //Logic op.
      ALU_OP_SHIFT: alu_result = shift_result;              //Shift op.
   endcase
end

//Driving the result output.
assign result = alu_result[7:0];


//******************************************************************************
//* Zero flag (Z).                                                             *
//* The zero flag is updated when an ALU operation except move is executed.    *
//******************************************************************************
always @(posedge clk)
begin
   if (op_type != ALU_OP_MOVE)
      flag_z <= (result == 8'd0);
end


//******************************************************************************
//* Carry flag (C).                                                            *
//* The carry flag is updated when an arithmetic or shift op. is executed.     *
//******************************************************************************
always @(posedge clk)
begin
   if ((op_type == ALU_OP_ARITH) || (op_type == ALU_OP_SHIFT))
      flag_c <= alu_result[8];
end


endmodule
