//******************************************************************************
//* The control unit of the MiniCISC CPU.                                      *
//******************************************************************************
module control_unit(
   //Clock and reset.
   input  wire        clk,             //Clock signal
   input  wire        rst,             //Reset signal
   
   //Value of the program counter.
   output wire [7:0]  pc,
   
   //Signals related to the memory.
   output wire [1:0]  mem_addr_sel,    //Address select signal
   input  wire [7:0]  mem_din,         //Input data
   output wire        mem_wr,          //Write enable signal
   output wire        mem_rd,          //Read enable signal
   
   //Constant data from the instruction.
   output wire        alu_shift_in,    //Bit to be shifted in.
   
   //Multiplexer control signals.
   output wire [1:0]  alu_op1_sel,     //ALU operand 1 select signal
   output wire        alu_op2_sel,     //ALU operand 2 select signal
   
   //Signals related to the registers.
   output wire [1:0]  reg_wr_en,       //A, X and Y reg. write enable signal
   output wire        addr_reg_en,     //Abs. addr. register write enable
   output wire        dout_reg_en,     //Output data register write enable
   input  wire [7:0]  x_reg,           //Value of the X index register
   input  wire [7:0]  y_reg,           //Value of the Y index register
   input  wire [7:0]  data_in_reg,     //Value of the input data register
   
   //Signals related to the ALU.
   output wire [1:0]  alu_op_type,     //Operation type select signal
   output wire        alu_add_sub,     //Arithmetic operation select signal
   output wire [1:0]  alu_logic_op,    //Logic operation select signal
   output wire [1:0]  alu_shift_op,    //Shift operation select signal
   input  wire        alu_flag_z,      //Zero flag
   input  wire        alu_flag_c,      //Carry flag
   
   //Signals for the display unit.
   output wire [7:0]  next_pc,         //Next value of the program counter
   //output wire [3:0]  ctrl_state,      //State of the controller (E, D, F, I)
   output wire [15:0] ir_value         //Value of the instruction register
   //output wire        ex_prg_ctrl      //Program control instruction is executed
);

`include "src\opcode_defs.vh"

//******************************************************************************
//* Instantiating the program counter (PC).                                    *
//******************************************************************************
reg  [7:0] pc_jmp_addr;    //Jump address
wire       pc_clr;         //PC clear signal
wire       pc_jmp;         //PC load signal (jump, subrutine call/return)
wire       pc_rts;         //Return address select (return from subrutine)
wire       pc_jsr;         //Save the return address (subrutine call)
wire       pc_inc;         //PC enable signal

program_counter prog_cnt(
   //Clock and reset.
   .clk(clk),              //Clock signal
   .pc_clr(pc_clr),        //Reset signal
   
   //Control signals.
   .pc_jmp(pc_jmp),        //Load signal (jump, subrutine call/return)
   .pc_rts(pc_rts),        //Return address select (return from subrutine)
   .pc_jsr(pc_jsr),        //Save the return address (subrutine call)
   .pc_inc(pc_inc),        //Enable signal
   
   //Jump address.
   .jmp_addr(pc_jmp_addr),
   
   //Value of the program counter.
   .pc(pc),                //Current value of the program counter
   .next_pc(next_pc)       //Next value of the program counter
);


//******************************************************************************
//* Instruction register (IR).                                                 * 
//* The instruction register stores the instruction read from the memory.      *
//* See the 'opcode_defs.vh' file for the instruction format.                  *
//******************************************************************************
reg  [7:0] ir;            //Instruction register
wire       ir_ld;         //Instruction register load signal

always @(posedge clk)
begin
   if (ir_ld)
      ir <= mem_din;
end

assign ir_value = ir;

//New value of the program counter (jump address).
always @(*)
begin
   case (ir[7:6])
      AMODE_IMM_ACC  : pc_jmp_addr = data_in_reg;
      AMODE_ABSOLUTE : pc_jmp_addr = data_in_reg;
      AMODE_INDIRECTX: pc_jmp_addr = x_reg;
      AMODE_INDIRECTY: pc_jmp_addr = y_reg;
   endcase
end

//Bit to be shifted in.
assign alu_shift_in = ir[1];


//******************************************************************************
//* Instantiating the controller state machine.                                *
//******************************************************************************
controller_fsm controller(
   //Clock and reset.
   .clk(clk),                       //Clock signal
   .rst(rst),                       //Reset signal
   
   //Signals related to the instruction register.
   .ir(ir),                         //Value of the instruction register
   .ir_ld(ir_ld),                   //Instruction register load signal
   
   //Signals related to the program counter.
   .pc_clr(pc_clr),                 //PC clear signal
   .pc_jmp(pc_jmp),                 //PC load signal (jump, subrutine call/return)
   .pc_rts(pc_rts),                 //Return address select (return from subrutine)
   .pc_jsr(pc_jsr),                 //Save the return address (subrutine call)
   .pc_inc(pc_inc),                 //PC increment signal
   
   //Signals related to the datapath.
   .alu_op1_sel(alu_op1_sel),       //ALU operand 1 select signal
   .alu_op2_sel(alu_op2_sel),       //ALU operand 2 select signal
   .reg_wr_en(reg_wr_en),           //A, X and Y reg. write enable signal
   .addr_reg_en(addr_reg_en),       //Abs. addr. register write enable
   .dout_reg_en(dout_reg_en),       //Output data register write enable
   .alu_op_type(alu_op_type),       //Operation type select signal
   .alu_add_sub(alu_add_sub),       //Arithmetic operation select signal
   .alu_logic_op(alu_logic_op),     //Logic operation select signal
   .alu_shift_op(alu_shift_op),     //Shift operation select signal
   .alu_flag_z(alu_flag_z),         //Zero flag
   .alu_flag_c(alu_flag_c),         //Carry flag
   
   //Signals related to the data memory.
   .mem_addr_sel(mem_addr_sel),     //Address select signal
   .mem_wr(mem_wr),                 //Write enable signal
   .mem_rd(mem_rd)                  //Read enable signal
   
   //Signals for the display unit.
   //.ctrl_state(ctrl_state),         //State of the controller (E, D, F, I)
   //.ex_prg_ctrl(ex_prg_ctrl)        //Program control instruction is executed
);


endmodule
