//******************************************************************************
//* MiniCISC CPU                                                               *
//*                                                                            *
//* The controller state machine of the processor.                             *
//******************************************************************************
module controller_fsm(
   //Clock and reset.
   input  wire        clk,                //Clock signal
   input  wire        rst,                //Reset signal
   
   //Signals related to the instruction register.
   input  wire [7:0]  ir,                 //Value of the instruction register
   output wire        ir_ld,              //Instruction register load signal
   
   //Signals related to the program counter.
   output wire        pc_clr,             //PC clear signal
   output reg         pc_jmp,             //PC load signal (jump, subrutine call/return)
   output wire        pc_rts,             //Return address select (return from subrutine)
   output wire        pc_jsr,             //Save the return address (subrutine call)
   output reg         pc_inc,             //PC increment signal
   
   //Signals related to the datapath.
   output reg  [1:0]  alu_op1_sel,        //ALU operand 1 select signal
   output wire        alu_op2_sel,        //ALU operand 2 select signal
   output reg  [1:0]  reg_wr_en,          //A, X and Y reg. write enable signal
   output wire        addr_reg_en,        //Abs. addr. register write enable
   output wire        dout_reg_en,        //Output data register write enable
   output reg  [1:0]  alu_op_type,        //Operation type select signal
   output wire        alu_add_sub,        //Arithmetic operation select signal
   output wire [1:0]  alu_logic_op,       //Logic operation select signal
   output wire [1:0]  alu_shift_op,       //Shift operation select signal
   input  wire        alu_flag_z,         //Zero flag
   input  wire        alu_flag_c,         //Carry flag
   
   //Signals related to the data memory.
   output reg  [1:0]  mem_addr_sel,       //Address select signal
   output wire        mem_wr,             //Write enable signal
   output wire        mem_rd              //Read enable signal
   
   //Signals for the display unit.
   //output wire [3:0]  ctrl_state,         //State of the controller (E, D, F, I)
   //output wire        ex_prg_ctrl         //Program control instruction is executed
);

`include "src\control_defs.vh"
`include "src\opcode_defs.vh"

//******************************************************************************
//* Generating some widely used signals.                                       *
//******************************************************************************
//Signals related to the addressing mode.
wire [1:0] amode            = ir[7:6];
wire       amode_is_imm_acc = (amode == AMODE_IMM_ACC);

//Signals related to the operation type.
wire [1:0] optype           = ir[5:4];
wire       optype_is_move   = (optype == OPTYPE_MOVE);
wire       optype_is_alu_pc = (optype == OPTYPE_ALU_PC);
wire       optype_is_alu1op = optype_is_alu_pc & ~ir[3];
wire       optype_is_pctrl  = optype_is_alu_pc &  ir[3];

//Signals related to the move operations.
wire [1:0] move_src         = ir[1:0];
wire [1:0] move_dst         = ir[3:2];
wire       move_src_mem     = (move_src == DATA_SRC_M);
wire       move_dst_mem     = (move_dst == DATA_DST_M);

//Signals related to the arithmetic operations.
wire [1:0] arith_op1_dst    = ir[3:2];
wire       discard_result   = ir[1];

//Signals related to the unary ALU operations.
wire [2:0] alu1op_type      = ir[2:0];
wire       alu1op_is_arith  = (alu1op_type == OP_INC) | (alu1op_type == OP_DEC);

//Signals related to the program control operations.
wire [2:0] pctrl_type       = ir[2:0];


//******************************************************************************
//* Controller state machine.                                                  *
//******************************************************************************
localparam STATE_INIT    = 3'd0;   //Initialization
localparam STATE_FETCH_I = 3'd1;   //Instruction fetch
localparam STATE_DECODE  = 3'd2;   //Instruction decode
localparam STATE_FETCH_O = 3'd3;   //Operand fetch
localparam STATE_EX_ALU  = 3'd4;   //Instruction execute: ALU operation
localparam STATE_EX_CTRL = 3'd5;   //Instruction execute: program control
localparam STATE_MEM_WR  = 3'd6;   //Memory write

//Register that stores the current state.
reg [2:0] current_state;

always @(posedge clk or posedge rst)
begin
   if (rst)
      current_state <= STATE_INIT;
   else
      case (current_state)
         //Initialization.
         STATE_INIT   : current_state <= STATE_FETCH_I;
         
         //Instruction fetch: the opcode is read from the memory.
         STATE_FETCH_I: current_state <= STATE_DECODE;
         
         //Instruction decode: the opcode is examined and the constant or
         //the absolute address is fetched from the memory (discarded and
         //the PC is not incremented if not used). If the operand is not
         //available in this state, it will be fetched in the next state.
         STATE_DECODE : if (amode_is_imm_acc)
                           if (optype_is_pctrl)
                              current_state <= STATE_EX_CTRL;  //Ctrl. op.: immediate
                           else
                              current_state <= STATE_EX_ALU;   //ALU op.  : imm./acc. 
                        else
                           if (optype_is_move && move_dst_mem)
                              current_state <= STATE_EX_ALU;   //Write-only operation
                           else
                              current_state <= STATE_FETCH_O;  //Fetch operand from memory
                            
         //Operand fetch: fetch the operand from the memory.
         STATE_FETCH_O: if (optype_is_pctrl)
                            current_state <= STATE_EX_CTRL;    //Program control operation
                         else
                            current_state <= STATE_EX_ALU;     //ALU operation
                            
         //Instruction execute (ALU operation).
         STATE_EX_ALU  : if (optype_is_alu_pc && (amode_is_imm_acc == 0))
                            current_state <= STATE_MEM_WR;     //Unary ALU op. on memory
                         else
                            if (optype_is_move && move_dst_mem)
                               current_state <= STATE_MEM_WR;  //Move reg. to memory
                            else
                               current_state <= STATE_FETCH_I; //No memory write is necessary
         
         //Instruction execute (program control operation).
         STATE_EX_CTRL : current_state <= STATE_FETCH_I;
         
         //Memory write.
         STATE_MEM_WR  : current_state <= STATE_FETCH_I;
         
         //Invalid states.
         default       : current_state <= STATE_INIT;
      endcase
end


//******************************************************************************
//* Driving the instruction register control signal.                           *
//******************************************************************************
//The instruction register is loaded in the instruction fetch state.
assign ir_ld = (current_state == STATE_FETCH_I);


//******************************************************************************
//* Driving the program counter control signals.                               *
//******************************************************************************
//The program counter is cleared in the initialization state.
assign pc_clr = (current_state == STATE_INIT);

//The program counter is incremented in the instruction fetch
//state and optionally in the instruction decode state.
always @(*)
begin
   case (current_state)
      //Instruction fetch: the PC is incremented.
      STATE_FETCH_I: pc_inc = 1'b1;
      
      //Instruction decode: the PC is incremented if the source
      //or the destination of the operation is the memory and not
      //indirect addressing mode is used. In this case the constant
      //or the absolute address is fetched in the decode state.
      STATE_DECODE : if (optype_is_move)
                        if (move_src_mem || move_dst_mem)
                           case (amode)
                              AMODE_IMM_ACC : pc_inc = 1'b1;   //Move: immediate
                              AMODE_ABSOLUTE: pc_inc = 1'b1;   //Move: absolute addr.
                              default       : pc_inc = 1'b0;   //Move: indirect addr.
                           endcase
                        else
                           pc_inc = 1'b0;                      //Move: between registers
                     else
                        case (amode)
                           AMODE_IMM_ACC : if (optype_is_alu1op)
                                              pc_inc = 1'b0;   //Unary ALU op. on acc.
                                           else
                                              pc_inc = 1'b1;   //Immediate addressing
                           AMODE_ABSOLUTE: pc_inc = 1'b1;      //Absolute addressing
                           default       : pc_inc = 1'b0;      //Indirect addressing
                        endcase
      
      //Other states: the PC is not incremented.
      default      : pc_inc = 1'b0;
   endcase
end

//The program counter is loaded with the new value when the
//program control instructions (except NOP) are executed.
always @(*)
begin
   if (current_state == STATE_EX_CTRL)
      case (pctrl_type)
         PRG_CTRL_JZ : pc_jmp = alu_flag_z;     //Jump if Z is 1
         PRG_CTRL_JNZ: pc_jmp = ~alu_flag_z;    //Jump if Z is 0
         PRG_CTRL_JC : pc_jmp = alu_flag_c;     //Jump if C is 1
         PRG_CTRL_JNC: pc_jmp = ~alu_flag_c;    //Jump if C is 0
         PRG_CTRL_NOP: pc_jmp = 1'b0;           //No operation
         default     : pc_jmp = 1'b1;           //Unconditional jump
      endcase
   else
      pc_jmp = 1'b0;
end

//Save the current program counter value in case of a subrutine call.
assign pc_jsr = (current_state == STATE_EX_CTRL) & (pctrl_type == PRG_CTRL_JSR);

//Load the program counter with the return address when an RTS is executed.
assign pc_rts = (current_state == STATE_EX_CTRL) & (pctrl_type == PRG_CTRL_RTS);


//******************************************************************************
//* Driving the internal datapath control signals.                             *
//******************************************************************************
//ALU operand 1 select signal.
always @(*)
begin
   case (optype)
      //Move operation: ALU operand 1 is the data source.
      OPTYPE_MOVE  : alu_op1_sel = move_src;
      
      //Binary arithmetic operation: ALU operand 1 is the data source.
      OPTYPE_ARITH : alu_op1_sel = arith_op1_dst;
      
      //Binary logic operation: ALU operand 1 is accumulator.
      OPTYPE_LOGIC : alu_op1_sel = DATA_SRC_A;
      
      //Unary ALU operation: ALU operand 1 depends on the addressing mode.
      OPTYPE_ALU_PC: if (amode_is_imm_acc)
                        alu_op1_sel = DATA_SRC_A;
                     else
                        alu_op1_sel = DATA_SRC_M;
   endcase
end

//ALU operand 2 select signal. Constant 1 is selected in case of
//unary arithmetic (INC and DEC) operations.
assign alu_op2_sel = optype_is_alu_pc & alu1op_is_arith;

//Register write enable signal.
always @(*)
begin
   if (current_state == STATE_EX_ALU)
      case (optype)
         //Move operation: the ALU result is written to the destination reg.
         OPTYPE_MOVE  : reg_wr_en = move_dst;
         
         //Binary arithmetic operation: the ALU result is written to the
         //destination register or discarded.
         OPTYPE_ARITH : if (discard_result == 0)
                           reg_wr_en = arith_op1_dst;
                        else
                           reg_wr_en = DATA_DST_M;    //No register write
         
         //Binary logic operation: the result is written to the accumulator
         //or discarded.
         OPTYPE_LOGIC : if (discard_result == 0)
                           reg_wr_en = DATA_DST_A;
                        else
                           reg_wr_en = DATA_DST_M;    //No register write
         
         //Unary ALU operation: the destination reg. depends on the addr. mode.
         OPTYPE_ALU_PC: if (amode_is_imm_acc)
                           reg_wr_en = DATA_DST_A;
                        else
                           reg_wr_en = DATA_DST_M;    //No register write
      endcase
   else
      reg_wr_en = DATA_DST_M;                         //No register write
end

//Absolute address register write enable.
assign addr_reg_en = (current_state == STATE_DECODE);

//Output data register write enable signal.
assign dout_reg_en = (current_state == STATE_EX_ALU);
   
//ALU operation type select signal.
always @(*)
begin
   if (current_state == STATE_EX_ALU)
      case (optype)
         //Move operation.
         OPTYPE_MOVE  : alu_op_type = ALU_OP_MOVE;
         
         //Binary arithmetic operation
         OPTYPE_ARITH : alu_op_type = ALU_OP_ARITH;
         
         //Binary logic operation
         OPTYPE_LOGIC : alu_op_type = ALU_OP_LOGIC;
         
         //Unary ALU operation: arithmetic or shift
         OPTYPE_ALU_PC: if (alu1op_is_arith)
                           alu_op_type = ALU_OP_ARITH;
                        else
                           alu_op_type = ALU_OP_SHIFT;
      endcase
   else
      alu_op_type = ALU_OP_MOVE;
end

//Arithmetic operation select signal.
assign alu_add_sub  = ir[0];

//Logic operation select signal.
assign alu_logic_op = ir[3:2];

//Shift operation select signal.
assign alu_shift_op = {ir[2], ir[0]};


//******************************************************************************
//* Driving the data memory control signals.                                   *
//******************************************************************************
//Memory address select signal. Address other than the program counter
//can be selected in the operand fetch and memory write states.
always @(*)
begin
   if ((current_state == STATE_FETCH_O) || (current_state == STATE_MEM_WR))
      case (amode)
         AMODE_IMM_ACC  : mem_addr_sel = MEM_ADDR_PC;
         AMODE_ABSOLUTE : mem_addr_sel = MEM_ADDR_ABS;
         AMODE_INDIRECTX: mem_addr_sel = MEM_ADDR_X;
         AMODE_INDIRECTY: mem_addr_sel = MEM_ADDR_Y;
      endcase
   else
      mem_addr_sel = MEM_ADDR_PC;
end

//Memory write enable signal.
assign mem_wr = (current_state == STATE_MEM_WR);

//Memory read enable signal.
assign mem_rd = (current_state == STATE_FETCH_I) |
                (current_state == STATE_DECODE)  |
                (current_state == STATE_FETCH_O);


//******************************************************************************
//* Driving the signals for the display unit.                                  *
//******************************************************************************
//Current state of the controller.
//assign ctrl_state[0] = (current_state == STATE_INIT);       //Init
//assign ctrl_state[1] = (current_state == STATE_FETCH);      //Fetch
//assign ctrl_state[2] = (current_state == STATE_DECODE);     //Decode
//assign ctrl_state[3] = (ctrl_state[2:0] == 3'b000);         //Execute

//Indicates if a program control instruction is executed.
//assign ex_prg_ctrl   = (current_state == STATE_EX_CTRL);       


endmodule
