//******************************************************************************
//* MiniCISC CPU                                                               *
//*                                                                            *
//* The datapath of the processor.                                             *
//******************************************************************************
module datapath(
   //Clock signal.
   input  wire       clk,
   
   //Signals related to the memory.
   input  wire [7:0] mem_din,          //Input data
   output reg  [7:0] mem_dout,         //Output data
   
   //Constant data from the instruction.
   input  wire       alu_shift_in,     //Bit to be shifted in
   
   //Multiplexer control signals.
   input  wire [1:0] alu_op1_sel,      //ALU operand 1 select signal
   input  wire       alu_op2_sel,      //ALU operand 2 select signal
   
   //Signals related to the registers.
   input  wire [1:0] reg_wr_en,        //A, X and Y reg. write enable signal
   input  wire       dout_reg_en,      //Output data register write enable
   output reg  [7:0] x_reg,            //Value of the X index register
   output reg  [7:0] y_reg,            //Value of the Y index register
   output reg  [7:0] data_in_reg,      //Value of the input data register
   
   //Signals related to the ALU.
   input  wire [1:0] alu_op_type,      //Operation type select signal
   input  wire       alu_add_sub,      //Arithmetic operation select signal
   input  wire [1:0] alu_logic_op,     //Logic operation select signal
   input  wire [1:0] alu_shift_op,     //Shift operation select signal
   output wire       alu_flag_z,       //Zero flag
   output wire       alu_flag_c        //Carry flag
);

`include "src\control_defs.vh"
`include "src\opcode_defs.vh"

//******************************************************************************
//* Accumulator register.                                                      *
//******************************************************************************
reg  [7:0] accumulator;
wire [7:0] alu_result;

always @(posedge clk)
begin
   if (reg_wr_en == DATA_DST_A)
      accumulator <= alu_result;
end


//******************************************************************************
//* X index register.                                                          *
//******************************************************************************
always @(posedge clk)
begin
   if (reg_wr_en == DATA_DST_X)
      x_reg <= alu_result;
end


//******************************************************************************
//* Y index register.                                                          *
//******************************************************************************
always @(posedge clk)
begin
   if (reg_wr_en == DATA_DST_Y)
      y_reg <= alu_result;
end


//******************************************************************************
//* Input data register (stores the data read from the memory).                *
//******************************************************************************
always @(posedge clk)
begin
   data_in_reg <= mem_din;
end


//******************************************************************************
//* ALU operand select multiplexers.                                           *
//******************************************************************************
reg  [7:0] alu_operand1;
wire [7:0] alu_operand2 = (alu_op2_sel) ? 8'd1 : data_in_reg;

always @(*)
begin
   case (alu_op1_sel)
      DATA_SRC_A: alu_operand1 = accumulator;
      DATA_SRC_M: alu_operand1 = data_in_reg;
      DATA_SRC_X: alu_operand1 = x_reg;
      DATA_SRC_Y: alu_operand1 = y_reg;
   endcase
end


//******************************************************************************
//* Instantiating the arithmetic and logic unit (ALU).                         *
//******************************************************************************
alu alu(
   //Clock signal.
   .clk(clk),
   
   //Signals related to the function selection.
   .op_type(alu_op_type),        //Operation type select signal
   .add_sub(alu_add_sub),        //Arithmetic operation select signal
   .logic_op(alu_logic_op),      //Logic operation select signal
   .shift_op(alu_shift_op),      //Shift operation select signal
   
   //Input operands.
   .operand1(alu_operand1),      //ALU operand 1
   .operand2(alu_operand2),      //ALU operand 2
   .shift_in(alu_shift_in),      //Bit to be shifted in.
   
   //ALU result.
   .result(alu_result),
   
   //Flags.
   .flag_z(alu_flag_z),          //Zero flag
   .flag_c(alu_flag_c)           //Carry flag
);


//******************************************************************************
//* Output data register (stores the ALU result for memory writes).            *
//******************************************************************************
always @(posedge clk)
begin
   if (dout_reg_en)
      mem_dout <= alu_result;
end


endmodule
