//******************************************************************************
//* 256 x 8 bit memory. Can be updated through the JTAG interface.             *
//******************************************************************************
module memory(
   input  wire       clk,           //Clock signal
   input  wire [7:0] address,       //Address input
   input  wire       write_en,      //Write enable signal
   input  wire [7:0] data_in,       //Data input
   output wire [7:0] data_out,      //Data output
   output wire       jtag_rst       //Reset output (reset during JTAG update)
);

//******************************************************************************
//* JTAG interface.                                                            *
//******************************************************************************
wire        drck1;
wire        update;
wire        mem_clk;
wire        shr_clk;
reg  [15:0] shr;
wire        shr_din;

//Boundary scan interface.
BSCAN_SPARTAN3 bscan_interface(
   .CAPTURE(),          //CAPTURE output from TAP controller
   .DRCK1(drck1),       //Data register output for USER1 functions
   .DRCK2(),            //Data register output for USER2 functions
   .RESET(),            //Reset output from TAP controller
   .SEL1(jtag_rst),     //USER1 active output
   .SEL2(),             //USER2 active output
   .SHIFT(),            //SHIFT output from TAP controller
   .TDI(shr_din),       //TDI output from TAP controller
   .UPDATE(update),     //UPDATE output from TAP controller
   .TDO1(1'b0),         //Data input for USER1 function
   .TDO2(1'b0)          //Data input for USER2 function
);

//Memory clock select multiplexer. BUFGMUX cannot be used in this case
//because it requires continous clocks to switch between the sources.
wire mem_clk_in = (jtag_rst) ? update : clk;

//Clock buffers.
BUFG shr_clk_bufg(.I(drck1), .O(shr_clk));
BUFG mem_clk_bufg(.I(mem_clk_in), .O(mem_clk));

//Shift register.
always @(posedge shr_clk)
begin
   shr <= {shr_din, shr[15:1]};
end


//******************************************************************************
//* 256 x 8 bit distributed RAM.                                               *
//******************************************************************************
(* ram_style = "distributed" *)
reg  [7:0] mem [255:0];
wire [7:0] mem_addr = (jtag_rst) ? shr[15:8] : address;
wire [7:0] mem_din  = (jtag_rst) ? shr[7:0]  : data_in;
wire       mem_wr   = (jtag_rst) ? 1'b1      : write_en;

always @(posedge mem_clk)
begin
   if (mem_wr)
      mem[mem_addr] <= mem_din;
end

assign data_out = mem[mem_addr];


endmodule
