//******************************************************************************
//* MiniCISC CPU                                                               *
//*                                                                            *
//* Top level module of the processor.                                         *
//******************************************************************************
module minicisc_cpu(
   //Clock and reset.
   input  wire        cpu_clk,         //Clock signal for the processor
   input  wire        disp_clk,        //Clock signal for the display unit
   input  wire        rst,             //Reset signal
   
   //Signals related to the memory.
   output reg  [7:0]  mem_addr,        //Address
   output wire        mem_wr,          //Write enable signal
   output wire        mem_rd,          //Read enable signal
   input  wire [7:0]  mem_din,         //Input data
   output wire [7:0]  mem_dout,        //Output data
   
   //Signals related to the display unit.
   output wire [7:0]  seg,             //Segment data
   output wire [3:0]  dig,             //Digit select signals
   output wire [4:0]  col              //Column select signals
);

`include "src\control_defs.vh"

//******************************************************************************
//* Instantiating the control unit.                                            *
//******************************************************************************
wire [7:0] pc;
wire [1:0] mem_addr_sel;
wire       alu_shift_in;
wire [1:0] alu_op1_sel;
wire       alu_op2_sel;
wire [1:0] reg_wr_en;
wire       addr_reg_en;
wire       dout_reg_en;
wire [7:0] x_reg;
wire [7:0] y_reg;
wire [7:0] data_in_reg;
wire [1:0] alu_op_type;
wire       alu_add_sub;
wire [1:0] alu_logic_op;
wire [1:0] alu_shift_op;
wire       alu_flag_z;
wire       alu_flag_c;

//wire [7:0]  next_pc;
//wire [3:0]  ctrl_state;
//wire [15:0] ir_value;
//wire        ex_prg_ctrl;

control_unit control_unit(
   //Clock and reset.
   .clk(cpu_clk),                      //Clock signal
   .rst(rst),                          //Reset signal
   
   //Value of the program counter.
   .pc(pc),
   
   //Signals related to the memory.
   .mem_addr_sel(mem_addr_sel),        //Address select signal
   .mem_din(mem_din),                  //Input data
   .mem_wr(mem_wr),                    //Write enable signal
   .mem_rd(mem_rd),                    //Read enable signal
   
   //Constant data from the instruction.
   .alu_shift_in(alu_shift_in),        //Bit to be shifted in
   
   //Multiplexer control signals.
   .alu_op1_sel(alu_op1_sel),          //ALU operand 1 select signal
   .alu_op2_sel(alu_op2_sel),          //ALU operand 2 select signal
   
   //Signals related to the registers.
   .reg_wr_en(reg_wr_en),              //A, X and Y reg. write enable signal
   .addr_reg_en(addr_reg_en),          //Abs. addr. register write enable
   .dout_reg_en(dout_reg_en),          //Output data register write enable
   .x_reg(x_reg),                      //Value of the X index register
   .y_reg(y_reg),                      //Value of the Y index register
   .data_in_reg(data_in_reg),          //Value of the input data register
   
   //Signals related to the ALU.
   .alu_op_type(alu_op_type),          //Operation type select signal
   .alu_add_sub(alu_add_sub),          //Arithmetic operation select signal
   .alu_logic_op(alu_logic_op),        //Logic operation select signal
   .alu_shift_op(alu_shift_op),        //Shift operation select signal
   .alu_flag_z(alu_flag_z),            //Zero flag
   .alu_flag_c(alu_flag_c)             //Carry flag
   
   //Signals for the display unit.
   //.next_pc(next_pc),                  //Next value of the program counter
   //.ctrl_state(ctrl_state),            //State of the controller (E, D, F, I)
   //.ir_value(ir_value),                //Value of the instruction register
   //.ex_prg_ctrl(ex_prg_ctrl)           //Program control instruction is executed
);


//******************************************************************************
//* Register that stores the absolute address.                                 *
//******************************************************************************
reg [7:0] abs_addr_reg;

always @(posedge cpu_clk)
begin
   if (addr_reg_en)
      abs_addr_reg <= mem_din;
end


//******************************************************************************
//* Memory address multiplexer.                                                *
//******************************************************************************
always @(*)
begin
   case (mem_addr_sel)
      MEM_ADDR_PC : mem_addr = pc;              //Program counter
      MEM_ADDR_ABS: mem_addr = abs_addr_reg;    //Absolute address
      MEM_ADDR_X  : mem_addr = x_reg;           //X index register
      MEM_ADDR_Y  : mem_addr = y_reg;           //Y index register
   endcase
end


//******************************************************************************
//* Instantiating the datapath.                                                *
//******************************************************************************
datapath datapath(
   //Clock signal.
   .clk(cpu_clk),
   
   //Signals related to the memory.
   .mem_din(mem_din),                  //Input data
   .mem_dout(mem_dout),                //Output data
   
   //Constant data from the instruction.
   .alu_shift_in(alu_shift_in),        //Bit to be shifted in.
   
   //Multiplexer control signals.
   .alu_op1_sel(alu_op1_sel),          //ALU operand 1 select signal
   .alu_op2_sel(alu_op2_sel),          //ALU operand 2 select signal
   
   //Signals related to the registers.
   .reg_wr_en(reg_wr_en),              //A, X and Y reg. write enable signal
   .dout_reg_en(dout_reg_en),          //Output data register write enable
   .x_reg(x_reg),                      //Value of the X index register
   .y_reg(y_reg),                      //Value of the Y index register
   .data_in_reg(data_in_reg),          //Value of the input data register
   
   //Signals related to the ALU.
   .alu_op_type(alu_op_type),          //Operation type select signal
   .alu_add_sub(alu_add_sub),          //Arithmetic operation select signal
   .alu_logic_op(alu_logic_op),        //Logic operation select signal
   .alu_shift_op(alu_shift_op),        //Shift operation select signal
   .alu_flag_z(alu_flag_z),            //Zero flag
   .alu_flag_c(alu_flag_c)             //Carry flag
);


//******************************************************************************
//* Instantiating the display unit.                                            *
//******************************************************************************
//display display(
//   //Clock and reset.
//   .clk(disp_clk),                     //Clock signal
//   .rst(rst),                          //Reset signal
//   
//   //Input signals.
//   .ctrl_state(ctrl_state),            //State of the controller (E, D, F, I)
//   .prg_mem_addr(prg_mem_addr),        //Program memory address
//   .ir_value(ir_value),                //Value of the instruction register
//   .next_pc(next_pc),                  //Next value of the program counter
//   .data_mem_addr(data_mem_addr),      //Data memory address
//   .data_mem_dout(data_mem_dout),      //Data memory output data
//   .reg_wr_data(reg_wr_data),          //Data to be written into the reg. file
//   .ex_load(data_mem_rd),              //Load instruction is executed
//   .ex_store(data_mem_wr),             //Store instruction is executed
//   .ex_prg_ctrl(ex_prg_ctrl),          //Program control instruction is executed
//   
//   //Display signals.
//   .seg(seg),                          //Segment data
//   .dig(dig),                          //Digit select signals
//   .col(col)                           //Column select signals
//);


endmodule
