//******************************************************************************
//* MiniRISC system top-level module.                                          *
//******************************************************************************
module minicisc_system(
   //Clock and reset.
   input  wire       clk16M,     //16 MHz clock signal
   input  wire       dev_clk,    //Clock signal from the dev. port
   input  wire       rstbt,      //Reset signal from the reset button
   input  wire       dev_rst,    //Reset signal from the dev. port
   
   //Peripherals.
   input  wire [7:0] sw,         //DIP switch
   input  wire [3:0] bt,         //Push-buttons
   output wire [7:0] ld,         //LEDs
   
   //Poers related to the displays.
   output wire [7:0] seg_n,      //Segment data  (active low)
   output wire [3:0] dig_n,      //Digit select  (active low)
   output wire [4:0] col_n       //Column select (active low)   
);

//******************************************************************************
//* Clock and reset.                                                           *
//******************************************************************************
wire jtag_rst;

wire clk = dev_clk;
wire rst = dev_rst | rstbt | jtag_rst;


//******************************************************************************
//* Instantiating the MiniRISC CPU.                                            *
//******************************************************************************
wire [7:0]  cpu2mem_addr;
wire        cpu2mem_wr;
wire [7:0]  cpu2mem_data;
reg  [7:0]  mem2cpu_data;

wire [7:0]  seg;
wire [3:0]  dig;
wire [4:0]  col;

minicisc_cpu minicisc_cpu(
   //Clock and reset.
   .cpu_clk(clk),                      //Clock signal for the processor
   .disp_clk(clk16M),                  //Clock signal for the display unit
   .rst(rst),                          //Reset signal
   
   //Signals related to the memory.
   .mem_addr(cpu2mem_addr),            //Address
   .mem_wr(cpu2mem_wr),                //Write enable signal
   .mem_rd(),                          //Read enable signal
   .mem_din(mem2cpu_data),             //Input data
   .mem_dout(cpu2mem_data),            //Output data
   
   //Signals related to the display unit.
   .seg(seg),                          //Segment data
   .dig(dig),                          //Digit select signals
   .col(col)                           //Column select signals
);

//Driving the display data and select signals.
assign seg_n = ~seg;
assign dig_n = ~dig;
assign col_n = ~col;


//******************************************************************************
//* Instantiating the memory.                                                  *
//* Address range: 0x00 - 0xEF (240 byte)                                      *
//******************************************************************************
wire [7:0] mem_dout;
wire       mem_wr = cpu2mem_wr & (cpu2mem_addr[7:4] != 4'hf);

memory memory(
   .clk(clk),                          //Clock signal
   .address(cpu2mem_addr),             //Address input
   .write_en(mem_wr),                  //Write enable signal
   .data_in(cpu2mem_data),             //Data input
   .data_out(mem_dout),                //Data output
   .jtag_rst(jtag_rst)                 //Reset output (reset during JTAG update) 
);


//******************************************************************************
//* LED peripheral.                                                            *
//* Address: 0xF0 (read/write)                                                 *
//******************************************************************************
reg  [7:0] led_reg;

//Write enable signal for the LED register.
wire       led_reg_wr  = cpu2mem_wr & (cpu2mem_addr[7:4] == 4'hf) & (cpu2mem_addr[1:0] == 2'b00);
//Input data for the LED register.
wire [7:0] led_reg_din = cpu2mem_data;

always @(posedge clk or posedge rst)
begin
   if (rst)
      led_reg <= 8'd0;
   else
      if (led_reg_wr)
         led_reg <= led_reg_din;
end

//Driving the LEDs.
assign ld = led_reg;


//******************************************************************************
//* DIP switch and buttons peripheral.                                         *
//* Address: 0xF1: DIP switch (read-only)                                      *
//*          0xF2: buttons    (read-only)                                      *
//******************************************************************************
reg [11:0] sw_bt_reg;

//Output data from the DIP switch.
wire [7:0]  sw_dout = sw_bt_reg[7:0];
//Output data from the buttons.
wire [7:0]  bt_dout = {4'd0, sw_bt_reg[11:8]};

always @(posedge clk or posedge rst)
begin
   if (rst)
      sw_bt_reg <= 12'd0;
   else
      sw_bt_reg <= {bt, sw};
end


//******************************************************************************
//* Driving the CPU input data bus.                                            *
//******************************************************************************
always @(*)
begin
   casex (cpu2mem_addr)
      8'b1111_xx00: mem2cpu_data = led_reg;
      8'b1111_xx01: mem2cpu_data = sw_dout;
      8'b1111_xx10: mem2cpu_data = bt_dout;
      8'b1111_xx11: mem2cpu_data = 8'd0;
      default     : mem2cpu_data = mem_dout;
   endcase
end


endmodule
