//******************************************************************************
//* MiniCISC CPU                                                               *
//*                                                                            *
//* Instruction definitions:                                                   *
//*              7      6   5    4   3      0                                  *
//* -1st byte: |addr. mode|op. type|op. select|                                *
//* -2nd byte: | absolute address / constant  | (optional)                     *
//******************************************************************************


//******************************************************************************
//* Addressing modes.                                                          *
//******************************************************************************
localparam AMODE_IMM_ACC   = 2'b00;    //Immediate or accumulator
localparam AMODE_ABSOLUTE  = 2'b01;    //Absolute
localparam AMODE_INDIRECTX = 2'b10;    //Indirect using the X register
localparam AMODE_INDIRECTY = 2'b11;    //Indirect using the Y register


//******************************************************************************
//* Data move operations.                                                      *
//*                                                                            *
//*    7      6  5  4 3  2 1  0                                                *
//*  |addr. mode| 00 |dst.|src.|                                               *
//*                    |    |                                                  *
//*                    |    \----> Data source: acc., X, Y or memory           *
//*                    \---------> Destination: acc., X, Y or memory           *
//******************************************************************************
localparam OPTYPE_MOVE = 2'b00;

localparam DATA_SRC_A = 2'b00;         //Source: accumulator
localparam DATA_SRC_M = 2'b01;         //Source: memory
localparam DATA_SRC_X = 2'b10;         //Source: X register
localparam DATA_SRC_Y = 2'b11;         //Source: Y register

localparam DATA_DST_A = 2'b00;         //Destination: accumulator
localparam DATA_DST_M = 2'b01;         //Destination: memory
localparam DATA_DST_X = 2'b10;         //Destination: X register
localparam DATA_DST_Y = 2'b11;         //Destination: Y register


//******************************************************************************
//* Binary arithmetic operations.                                              *
//*                                                                            *
//*    7      6  5  4  3    2  1  0                                            *
//*  |addr. mode| 01 |op1/dst.|D|A/S|                                          *
//*                       |    |  |                                            *
//*                       |    |  \--> Operation select   : add (0) / sub (1)  *
//*                       |    \-----> Discard the result : yes (1) / no (0)   * 
//*                       \----------> 1st op./destination: acc., X, or Y      *
//******************************************************************************
localparam OPTYPE_ARITH = 2'b01;

localparam OP_ADD = 2'b00;             //Addition
localparam OP_SUB = 2'b01;             //Subtraction
localparam OP_CMP = 2'b11;             //Compare


//******************************************************************************
//* Bitwise logic operations.                                                  *
//*                                                                            *
//*    7      6  5  4  3     2  1 0                                            *
//*  |addr. mode| 10 |operation|D|0|                                           *
//*                       |     |                                              *
//*                       |     \----> Discard the result: yes (1) / no (0)    *
//*                       \----------> Operation select  : AND, OR, XOR        *
//******************************************************************************
localparam OPTYPE_LOGIC = 2'b10;

localparam OP_AND = 4'b0000;           //Bitwise AND
localparam OP_TST = 4'b0010;           //Bit test
localparam OP_OR  = 4'b0100;           //Bitwise OR
localparam OP_XOR = 4'b1000;           //Bitwise XOR


//******************************************************************************
//* Unary ALU or program control operations.                                   *
//*                                                                            *
//*    7      6  5  4  3   2     0                                             *
//*  |addr. mode| 11 |A/P|operation|                                           *
//*                    |      |                                                *
//*                    |      \------> Operation select                        *
//*                    \-------------> ALU (0) / program control (1)           *
//******************************************************************************
localparam OPTYPE_ALU_PC = 2'b11;

localparam OP_SL0 = 3'b000;            //Shift left  (0 is shifted in)
localparam OP_SR0 = 3'b001;            //Shift right (1 is shifted in)
localparam OP_SL1 = 3'b010;            //Shift left  (0 is shifted in)
localparam OP_SR1 = 3'b011;            //Shift right (1 is shifted in)
localparam OP_ROL = 3'b100;            //Rotate left
localparam OP_ROR = 3'b101;            //Rotate right
localparam OP_INC = 3'b110;            //Increment
localparam OP_DEC = 3'b111;            //Decrement

localparam PRG_CTRL_JMP = 3'b000;      //Unconditional jump
localparam PRG_CTRL_JZ  = 3'b001;      //Jump if Z=1
localparam PRG_CTRL_JNZ = 3'b010;      //Jump if Z=0
localparam PRG_CTRL_JC  = 3'b011;      //Jump if C=1
localparam PRG_CTRL_JNC = 3'b100;      //Jump if C=0
localparam PRG_CTRL_JSR = 3'b101;      //Subrutine call
localparam PRG_CTRL_RTS = 3'b110;      //Return from subrutine
localparam PRG_CTRL_NOP = 3'b111;      //No operation
